package eu.jonahbauer.chat.bot.pizza.model.table;

import eu.jonahbauer.chat.bot.database.Entity;
import lombok.experimental.FieldNameConstants;
import org.jetbrains.annotations.NotNull;

import java.util.Locale;

import static eu.jonahbauer.chat.bot.pizza.util.FormatUtils.tabular;

@FieldNameConstants
public record Pizza(
        long id,
        long restaurantId,
        int number,
        @NotNull String name,
        @NotNull String ingredients,
        double price
) implements Entity<Pizza>, Comparable<Pizza> {

    public @NotNull String toPrettyString() {
        var number = tabular(number());
        var price = tabular(price(), 5);
        var name = name().toUpperCase(Locale.ROOT);
        var ingredients = ingredients();
        return STR."\{number} | \{price} € | \{name} \{ingredients}";
    }

    @Override
    public int compareTo(@NotNull Pizza o) {
        return Integer.compare(number, o.number);
    }
}
