package eu.jonahbauer.chat.bot.pizza.util;

import eu.jonahbauer.chat.bot.pizza.PizzaBotException;
import lombok.experimental.UtilityClass;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

@UtilityClass
public class ArgumentParser {
    public static @NotNull List<@NotNull String> parse(@NotNull String string) {
        var list = new ArrayList<String>();
        var quote = (char) 0;
        var escaped = false;

        var current = new StringBuilder();
        var it = string.chars().iterator();
        while (it.hasNext()) {
            var chr = (char) it.nextInt();
            if (escaped) {
                if (chr != '\n') current.append(chr);
            } else if (chr == '\\' && quote != '\'') {
                escaped = true;
            } else if (quote == 0 && chr == '"') {
                quote = '"';
            } else if (quote == 0 && chr == '\'') {
                quote = '\'';
            } else if (quote == chr) {
                quote = 0;
            } else if (chr == ' ' || chr == '\t') {
                if (!current.isEmpty()) {
                    list.add(current.toString());
                    current.setLength(0);
                }
            } else if (chr == '\n') {
                break;
            } else {
                current.append(chr);
            }
        }

        if (escaped) {
            throw new IllegalArgumentException("incomplete escape sequence");
        } else if (quote != 0) {
            throw new IllegalArgumentException("incomplete quote");
        }

        if (!current.isEmpty()) {
            list.add(current.toString());
        }

        return Collections.unmodifiableList(list);
    }

    public static boolean isLong(@NotNull String string) {
        try {
            Long.parseLong(string);
            return true;
        } catch (NumberFormatException _) {
            return false;
        }
    }

    public static long toLong(@NotNull String string) {
        try {
            return Long.parseLong(string);
        } catch (NumberFormatException ex) {
            throw new PizzaBotException(string + " ist keine gültige Zahl.");
        }
    }

    public static boolean isDouble(@NotNull String string) {
        try {
            Double.parseDouble(string);
            return true;
        } catch (NumberFormatException _) {
            return false;
        }
    }

    public static double toDouble(@NotNull String string) {
        try {
            return Double.parseDouble(string);
        } catch (NumberFormatException ex) {
            throw new PizzaBotException(string + " ist keine gültige Zahl.");
        }
    }
}
