CREATE TABLE "user"
(
    "id"   INTEGER PRIMARY KEY,
    "username" VARCHAR(255) NOT NULL,
    CONSTRAINT "uq_username" UNIQUE ("username")
);

CREATE TABLE "restaurant"
(
    "id"    INTEGER PRIMARY KEY AUTOINCREMENT,
    "name"  VARCHAR(255) NOT NULL,
    "city"  VARCHAR(255) NOT NULL,
    "phone" VARCHAR(255) NULL DEFAULT NULL,
    "notes" VARCHAR(255) NULL DEFAULT NULL
);

CREATE TABLE "pizza"
(
    "id"            INTEGER PRIMARY KEY AUTOINCREMENT,
    "restaurant_id" INTEGER      NOT NULL,
    "number"        INTEGER      NOT NULL,
    "name"          VARCHAR(255) NOT NULL,
    "ingredients"   VARCHAR(255) NOT NULL,
    "price"         VARCHAR(255) NOT NULL,
    CONSTRAINT "fk_pizza_restaurant" FOREIGN KEY ("restaurant_id") REFERENCES "restaurant" ("id") ON DELETE CASCADE,
    CONSTRAINT "uq_pizza_number" UNIQUE ("restaurant_id", "number")
);

CREATE TABLE "event"
(
    "id"       INTEGER PRIMARY KEY AUTOINCREMENT,
    "name"     VARCHAR(255) NOT NULL,
    "admin_id" INTEGER      NOT NULL,
    "active"   BOOLEAN      NOT NULL DEFAULT FALSE,
    CONSTRAINT "fk_event_admin" FOREIGN KEY ("admin_id") REFERENCES "user" ("id")
);

CREATE UNIQUE INDEX "uq_event_active" ON "event" ("active")
    WHERE "active" = TRUE;

CREATE TABLE "order"
(
    "id"            INTEGER PRIMARY KEY AUTOINCREMENT,
    "event_id"      INTEGER      NOT NULL,
    "restaurant_id" INTEGER      NOT NULL,
    "name"          VARCHAR(255) NOT NULL,
    "deadline"      TIMESTAMP    NOT NULL,
    "payment"       DOUBLE       NULL,
    CONSTRAINT "fk_order_event" FOREIGN KEY ("event_id") REFERENCES "event" ("id"),
    CONSTRAINT "fk_order_restaurant" FOREIGN KEY ("restaurant_id") REFERENCES "restaurant" ("id")
);

CREATE TABLE "order_item"
(
    "id"       INTEGER PRIMARY KEY AUTOINCREMENT,
    "order_id" INTEGER NOT NULL,
    "pizza_id" INTEGER NOT NULL,
    "notes"    VARCHAR(255),
    CONSTRAINT "fk_order_item_order" FOREIGN KEY ("order_id") REFERENCES "order" ("id") ON DELETE CASCADE,
    CONSTRAINT "fk_order_item_pizza" FOREIGN KEY ("pizza_id") REFERENCES "pizza" ("id")
);

CREATE TABLE "order_item_to_user"
(
    "order_item_id" INTEGER NOT NULL,
    "user_id"       INTEGER NOT NULL,
    "parts"         INTEGER NOT NULL DEFAULT 1,
    "payed"         BOOLEAN NOT NULL DEFAULT FALSE,
    PRIMARY KEY ("order_item_id", "user_id"),
    CONSTRAINT "fk_order_item_to_user_order_item" FOREIGN KEY ("order_item_id") REFERENCES "order_item" ("id") ON DELETE CASCADE,
    CONSTRAINT "fk_order_item_to_user_user_id" FOREIGN KEY ("user_id") REFERENCES "user" ("id")
);

CREATE VIEW "order_item_detail"
AS
SELECT "order_item"."id",
       "order_item"."notes",
       "order"."id"                   AS "order_id",
       "order"."name"                 AS "order_name",
       "order"."deadline"             AS "order_deadline",
       "order"."payment"              AS "order_payment",
       "order"."event_id"             AS "event_id",
       "restaurant"."id"              AS "restaurant_id",
       "restaurant"."name"            AS "restaurant_name",
       "pizza"."id"                   AS "pizza_id",
       "pizza"."number"               AS "pizza_number",
       "pizza"."name"                 AS "pizza_name",
       "pizza"."price"                AS "pizza_price",
       "order_item_to_user"."user_id" AS "user_id",
       "order_item_to_user"."payed"   AS "user_payed",
       "order_item_to_user"."parts"   AS "user_parts",
       "user"."username"              AS "user_name",
       (SELECT SUM("parts") FROM "order_item_to_user" WHERE "order_item_id" = "order_item"."id") AS "user_count"
FROM "order"
         JOIN "restaurant" ON "order"."restaurant_id" = "restaurant"."id"
         JOIN "order_item" ON "order"."id" = "order_item"."order_id"
         JOIN "pizza" ON "order_item"."pizza_id" = "pizza"."id"
         JOIN "order_item_to_user" ON "order_item"."id" = "order_item_to_user"."order_item_id"
         JOIN "user" ON "order_item_to_user"."user_id" = "user"."id";

CREATE VIEW "order_detail"
AS
SELECT "order"."id",
       "order"."name",
       "order"."deadline",
       "order"."payment",
       "event"."id" AS "event_id",
       "event"."name" AS "event_name",
       "event"."admin_id",
       "event"."active",
       "restaurant"."id" AS "restaurant_id",
       "restaurant"."name" AS "restaurant_name"
FROM "event"
    JOIN "order" ON "event"."id" = "order"."event_id"
    JOIN "restaurant" ON "order"."restaurant_id" = "restaurant"."id";